==> em_tasks-0.1.53-py2.py3-none-any.whl: em_tasks/tools/spree/sheet_orders_exporter.py <==
import datetime

from gspread.exceptions import APIError

from em_tasks import logger
from em_tasks.tools.spree.sheet_orders_base import SheetOrdersBase
from em_tasks.store.store_util import StoreUtil
from em_tasks.google.sheet_api import SheetAPI
from em_tasks.store.models import *


class SheetOrdersExporter(SheetOrdersBase):
  def __init__(self, service_account_data):
    self.service_account_data = service_account_data
    self.sheet_api = SheetAPI(service_account_data)

  def run(self):
    sheet_name = None
    default_sheet_name = (datetime.datetime.utcnow() - datetime.timedelta(hours=5)).strftime('%m/%d/%Y')
    old_sheet_name = (datetime.datetime.utcnow() - datetime.timedelta(hours=5)).strftime('%m/%d')
    spreadsheets = dict()
    orders = []
    for oi in OrderItem.select().where((OrderItem.risky == 0) & (OrderItem.exported == 0)).execute():
      orders.append(oi)

    for order in orders:
      sheet_order = self.convert_order_item_to_sheet_order(order)
      source = None
      spreadsheet_id = None
      if not order.vendor_id or not order.source:
        spreadsheet_id = self.__class__.spreadsheet_ids[1]['EMP']
      else:
        if order.vendor_id not in self.__class__.spreadsheet_ids:
          logger.debug(
            '[OrderNoSpreadsheet] OrderNumber: %s, OrderSKU: %s, SheetOrder: %s',
            order.s_order_number, order.s_sku, sheet_order)
          continue

        source = order.source[0].upper() + order.source[1:]
        if source not in self.__class__.spreadsheet_ids[order.vendor_id]:
          if order.vendor_id == 1:
            source = 'Other'
          else:
            logger.debug(
              '[OrderNoSpreadsheet] SellerID: %s, Source: %s, OrderNumber: %s, OrderSKU: %s, SheetOrder: %s',
              order.vendor_id, source, order.s_order_number, order.s_sku, sheet_order)
            continue

        spreadsheet_id = self.__class__.spreadsheet_ids[order.vendor_id][source]

      if not spreadsheet_id:
        continue

      if order.source and order.source.upper() == 'AMZ_US' and order.vendor_id == 24:
        sheet_name = old_sheet_name
      else:
        sheet_name = default_sheet_name

      if spreadsheet_id not in spreadsheets:
        spreadsheets[spreadsheet_id] = {'spreadsheet': self.sheet_api.get_spreadsheet(spreadsheet_id)}

      spreadsheet = spreadsheets[spreadsheet_id]['spreadsheet']

      spreadsheet_data = spreadsheets[spreadsheet_id]
      if sheet_name not in spreadsheet_data:
        spreadsheet_data[sheet_name] = self.sheet_api.find_or_create_sheet(spreadsheet, sheet_name)

      worksheet = spreadsheet_data[sheet_name]

      cols = None
      if not source:
        cols = self.sheet_cols['AMZ']
      elif source in self.sheet_cols:
        cols = self.sheet_cols[source]
      elif source.startswith('AMZ'):
        cols = self.sheet_cols['AMZ']
      elif source.startswith('Ebay'):
        cols = self.sheet_cols['Ebay']
      elif source.startswith('Rakuten') or source.startswith('rakuten'):
        cols = self.sheet_cols['Rakuten']
      elif source.startswith('EM'):
        cols = self.sheet_cols['EMP']
      elif source.startswith('Dangdang'):
        cols = self.sheet_cols['EMP']
      else:
        cols = self.sheet_cols['Other']

      record = dict()
      for idx, col in enumerate(cols):
        record[idx] = sheet_order.get(col, '')

      key = sheet_order['order-id']
      while True:
        try:
          self.sheet_api.append_row_by_key(worksheet, key, record)
          logger.info(
            '[OrderExported] OrderID: %s, Order: %s, SheetID: %s, SheetName: %s',
            key, sheet_order['sku'], spreadsheet_id, sheet_name)

          order.exported = True
          order.sheet_id = spreadsheet_id
          order.sheet_name = sheet_name
          order.save()

          break
        except APIError as e:
          logger.exception(e)
          logger.info('[WriteSheetThrottle] Wait 60s to retry')
          time.sleep(60)

  @classmethod
  def convert_order_item_to_sheet_order(self, order_item):
    remark = ''
    if order_item.canceled_at:
      remark = 'Cancelled'
    elif order_item.refund_qty:
      if order_item.refund_qty == order_item.s_quantity:
        remark = 'Refunded'
      else:
        remark = 'Partial Refunded'
    else:
      if not order_item.s_product_id or not order_item.s_variant_id:
        remark = 'ProductDeleted'
    return {
      'status': 'n',
      'order-id': '{}-{}'.format(order_item.s_order_id, order_item.s_line_item_id),
      'order-link': 'https://everymarket.com/admin/orders/{}'.format(order_item.s_order_number),
      'recipient-name': '{} {}'.format(order_item.first_name, order_item.last_name),
      'purchase-date': order_item.s_purchase_date.isoformat(),
      'sku address': order_item.s_url,
      'sku': '{}-{}'.format(order_item.s_order_number, order_item.s_sku),
      'price': order_item.s_price - (order_item.s_discount / order_item.s_quantity),
      'quantity-purchased': order_item.s_quantity - order_item.refund_qty,
      'shipping-fee': order_item.s_shipping_charge,
      'ship-state': order_item.state,
      'isbn-address': order_item.source_url,
      'isbn': order_item.source_id,
      'SELLER': '',
      'SELLER-ID': '',
      'SELLER PRICE': '',
      'URL': '',
      'condition': 'new',
      'character': '',
      'Remark': remark,
      'reference': '',
      'CODE': '',
      '0': '',
      'item-name': order_item.s_name,
      'ship-address-1': order_item.address1,
      'ship-address-2': order_item.address2 or '',
      'ship-city': order_item.city,
      'ship-zip': order_item.zipcode,
      'ship-phone-number': order_item.phone or '',
      'COST': '',
      'ORDER NUMBER': '',
      'ACCOUNT': '',
      'last code': '',
      'ship-country': order_item.country,
      'SID': '',
      'Sales Channel': '',
      'Condition': 'New-New',
      'Shipping Service': order_item.s_shipping_method or '',
      'Expected Ship Date': order_item.s_min_delivery_date or '',
      'Estimated Delivery Date': order_item.s_min_delivery_date or '',
      'Order Date': '',
      'Supplier Estimated Delivery Date': '',
      'Carrier': '',
      'Tracking Number': '',
      'Note1': '',
      'Note2': '',
      'Note3': '',
    }

==> em_tasks-0.1.61-py2.py3-none-any.whl: em_tasks/tools/spree/sheet_orders_exporter.py <==
import datetime

from gspread.exceptions import APIError

from em_tasks import logger
from em_tasks.tools.spree.sheet_orders_base import SheetOrdersBase
from em_tasks.store.store_util import StoreUtil
from em_tasks.google.sheet_api import SheetAPI
from em_tasks.store.models import *


class SheetOrdersExporter(SheetOrdersBase):
  def __init__(self, service_account_data):
    self.service_account_data = service_account_data
    self.sheet_api = SheetAPI(service_account_data)

  def run(self):
    sheet_name = None
    default_sheet_name = (datetime.datetime.utcnow() - datetime.timedelta(hours=5)).strftime('%m/%d/%Y')
    old_sheet_name = (datetime.datetime.utcnow() - datetime.timedelta(hours=5)).strftime('%m/%d')
    spreadsheets = dict()
    orders = []
    earliest_purchase_date = datetime.datetime.utcnow() - datetime.timedelta(days=10)
    for oi in OrderItem.select().where(OrderItem.exported == 0).execute():
      if oi.risky:
        continue

      if oi.s_purchase_date < earliest_purchase_date:
        continue

      orders.append(oi)

    for order in orders:
      sheet_order = self.convert_order_item_to_sheet_order(order)
      source = None
      spreadsheet_id = None
      if not order.vendor_id or not order.source:
        spreadsheet_id = self.__class__.spreadsheet_ids[1]['EMP']
      else:
        if order.vendor_id not in self.__class__.spreadsheet_ids:
          logger.debug(
            '[OrderNoSpreadsheet] OrderNumber: %s, OrderSKU: %s, SheetOrder: %s',
            order.s_order_number, order.s_sku, sheet_order)
          continue

        source = order.source[0].upper() + order.source[1:]
        if source not in self.__class__.spreadsheet_ids[order.vendor_id]:
          if order.vendor_id == 1:
            source = 'Other'
          else:
            logger.debug(
              '[OrderNoSpreadsheet] SellerID: %s, Source: %s, OrderNumber: %s, OrderSKU: %s, SheetOrder: %s',
              order.vendor_id, source, order.s_order_number, order.s_sku, sheet_order)
            continue

        spreadsheet_id = self.__class__.spreadsheet_ids[order.vendor_id][source]

      if not spreadsheet_id:
        continue

      if order.source and order.source.upper() == 'AMZ_US' and order.vendor_id == 24:
        sheet_name = old_sheet_name
      else:
        sheet_name = default_sheet_name

      if spreadsheet_id not in spreadsheets:
        spreadsheets[spreadsheet_id] = {'spreadsheet': self.sheet_api.get_spreadsheet(spreadsheet_id)}

      spreadsheet = spreadsheets[spreadsheet_id]['spreadsheet']

      spreadsheet_data = spreadsheets[spreadsheet_id]
      if sheet_name not in spreadsheet_data:
        spreadsheet_data[sheet_name] = self.sheet_api.find_or_create_sheet(spreadsheet, sheet_name)

      worksheet = spreadsheet_data[sheet_name]

      cols = None
      if not source:
        cols = self.sheet_cols['AMZ']
      elif source in self.sheet_cols:
        cols = self.sheet_cols[source]
      elif source.startswith('AMZ'):
        cols = self.sheet_cols['AMZ']
      elif source.startswith('Ebay'):
        cols = self.sheet_cols['Ebay']
      elif source.startswith('Rakuten') or source.startswith('rakuten'):
        cols = self.sheet_cols['Rakuten']
      elif source.startswith('EM'):
        cols = self.sheet_cols['EMP']
      elif source.startswith('Dangdang'):
        cols = self.sheet_cols['EMP']
      else:
        cols = self.sheet_cols['Other']

      record = dict()
      for idx, col in enumerate(cols):
        record[idx] = sheet_order.get(col, '')

      key = sheet_order['order-id']
      while True:
        try:
          self.sheet_api.append_row_by_key(worksheet, key, record)
          logger.info(
            '[OrderExported] OrderID: %s, Order: %s, SheetID: %s, SheetName: %s',
            key, sheet_order['sku'], spreadsheet_id, sheet_name)

          order.exported = True
          order.sheet_id = spreadsheet_id
          order.sheet_name = sheet_name
          order.save()

          break
        except APIError as e:
          logger.exception(e)
          logger.info('[WriteSheetThrottle] Wait 60s to retry')
          time.sleep(60)

  @classmethod
  def convert_order_item_to_sheet_order(self, order_item):
    remark = ''
    if order_item.canceled_at:
      remark = 'Cancelled'
    elif order_item.refund_qty:
      if order_item.refund_qty == order_item.s_quantity:
        remark = 'Refunded'
      else:
        remark = 'Partial Refunded'
    else:
      if not order_item.s_product_id or not order_item.s_variant_id:
        remark = 'ProductDeleted'
    return {
      'status': 'n',
      'order-id': '{}-{}'.format(order_item.s_order_id, order_item.s_line_item_id),
      'order-link': 'https://everymarket.com/admin/orders/{}'.format(order_item.s_order_number),
      'recipient-name': '{} {}'.format(order_item.first_name, order_item.last_name),
      'purchase-date': order_item.s_purchase_date.isoformat(),
      'sku address': order_item.s_url,
      'sku': '{}-{}'.format(order_item.s_order_number, order_item.s_sku),
      'price': order_item.s_price - (order_item.s_discount / order_item.s_quantity),
      'quantity-purchased': order_item.s_quantity - order_item.refund_qty,
      'shipping-fee': order_item.s_shipping_charge,
      'ship-state': order_item.state,
      'isbn-address': order_item.source_url,
      'isbn': order_item.source_id,
      'SELLER': '',
      'SELLER-ID': '',
      'SELLER PRICE': '',
      'URL': '',
      'condition': 'new',
      'character': '',
      'Remark': remark,
      'reference': '',
      'CODE': '',
      '0': '',
      'item-name': order_item.s_name,
      'ship-address-1': order_item.address1,
      'ship-address-2': order_item.address2 or '',
      'ship-city': order_item.city,
      'ship-zip': order_item.zipcode,
      'ship-phone-number': order_item.phone or '',
      'COST': '',
      'ORDER NUMBER': '',
      'ACCOUNT': '',
      'last code': '',
      'ship-country': order_item.country,
      'SID': '',
      'Sales Channel': '',
      'Condition': 'New-New',
      'Shipping Service': order_item.s_shipping_method or '',
      'Expected Ship Date': order_item.s_min_delivery_date or '',
      'Estimated Delivery Date': order_item.s_min_delivery_date or '',
      'Order Date': '',
      'Supplier Estimated Delivery Date': '',
      'Carrier': '',
      'Tracking Number': '',
      'Note1': '',
      'Note2': '',
      'Note3': '',
    }
